@@ -1,6 +1,6 @@
 /*
- renattach 1.2.3 - Filter that renames/deletes dangerous email attachments
- Copyright (C) 2003-2005  Jem E. Berkes <jberkes@pc-tools.net>
+ renattach 1.2.4 - Filter that renames/deletes dangerous email attachments
+ Copyright (C) 2003-2006  Jem E. Berkes <jberkes@pc-tools.net>
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -75,11 +75,11 @@ int closepipe(FILE*, pid_t, int*);
 /* Global variables */
 struct config_opts configuration;
 FILE* tempfile = NULL;
-int verbose=0, justsettings=0, excode=CODE_OK;
+int loopguard=0, verbose=0, justsettings=0, excode=CODE_OK;
 int filtcount=0, ban_count=0, exe_count=0, del_count=0, ren_count=0;
 char messageid[MAXFIELD];
 
-static char shortopts[] = "abc:deghkp:rsvV";
+static char shortopts[] = "abc:deghklp:rsvV";
 static struct option long_options[] =
 {
 	{"all", no_argument, NULL, 'a'},
@@ -90,6 +90,7 @@ static struct option long_options[] =
 	{"goodlist", no_argument, NULL, 'g'},
 	{"help", no_argument, NULL, 'h'},
 	{"kill", no_argument, NULL, 'k'},
+	{"loop", no_argument, NULL, 'l'},
 	{"pipe", required_argument, NULL, 'p'},
 	{"rename", no_argument, NULL, 'r'},
 	{"settings", no_argument, NULL, 's'},
@@ -163,6 +164,7 @@ int main(int argc, char** argv)
 				printf(TXT_HELP_G);
 				printf(TXT_HELP_H);
 				printf(TXT_HELP_K);
+				printf(TXT_HELP_L);
 				printf(TXT_HELP_P);
 				printf(TXT_HELP_R);
 				printf(TXT_HELP_S);
@@ -179,6 +181,10 @@ int main(int argc, char** argv)
 				}
 				configuration.def_act = ACTION_KILL;
 				break;
+				
+			case 'l':
+				loopguard = 1;
+				break;
 
 			case 'r':
 				if (configuration.def_act)
@@ -307,19 +313,24 @@ int filter_pass(FILE* dest)
 	/* Read line at a time from input */
 	while (fgets(linebuf, sizeof(linebuf), stdin))
 	{
-		/* Watch for end of message headers, if using delete_exe or kill_exe features */
-		if ( ((configuration.delete_exe == OPTION_ENABLED) || (configuration.kill_exe == OPTION_ENABLED))
-			&& onheaders && ((*linebuf == '\n') || (*linebuf == '\r')) )
+		if (onheaders && ((*linebuf == '\n') || (*linebuf == '\r')) )
 		{
-			sigcheckcount = 10;	/* check binary signatures on next few lines */
 			onheaders = 0;
+			/* Watch for end of message headers, if using delete_exe or kill_exe features */
+			if ((configuration.delete_exe == OPTION_ENABLED) || (configuration.kill_exe == OPTION_ENABLED))
+				sigcheckcount = 10;	/* check binary signatures on next few lines */
 		}
 
 		/* Record message-id for logging purposes */
-		if (strncasecmp(linebuf, HEAD_MESSAGEID, sizeof(HEAD_MESSAGEID)-1)==0)
-		{
+		if (onheaders && (strncasecmp(linebuf, HEAD_MESSAGEID, sizeof(HEAD_MESSAGEID)-1)==0))
 			strncpy(messageid, linebuf, sizeof(messageid)-1);
-			onheaders = 1;
+
+		/* Remove Delivered-To from headers to prevent spammer's mail forwarding loop trick */
+		if (onheaders && loopguard && (strncasecmp(linebuf, HEAD_DELIVERED_TO, sizeof(HEAD_DELIVERED_TO)-1)==0))
+		{
+			configuration.action = ACTION_SKIP;
+			taking_action(TXT_DELIVEREDTO);
+			continue;
 		}
 
 		/* First check for UUencode begin line */
@@ -1357,8 +1368,9 @@ void taking_action(const char* caughtfile)
 		closelog();
 		#endif
 	}
-	/* Update filter counters */
-	filtcount++;
+	/* Update filter counters (but not in case of ACTION_SKIP) */
+	if (configuration.action != ACTION_SKIP)
+		filtcount++;
 	if (configuration.action == ACTION_DELETE)
 		del_count++;
 	else if (configuration.action == ACTION_RENAME)
